using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace CovingtonImageProcessing
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        // Global variables have names starting with "the".

        public FormImage theFormImage;
        public FormReport theFormReport;
        public Bitmap theUndoImage;
        public DateTime theTime;

        #region Ready-made menu items

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // This version only opens one file at a time
            // so get rid of the client window if it exists
            if (theFormImage != null) theFormImage.Dispose();

            // Choose a file to open
            DialogResult r = openFileDialog1.ShowDialog();
            if (r != DialogResult.OK) return;

            // Set up the client window
            theFormImage = new FormImage();
            theFormImage.MdiParent = this;

            // Load and display the image
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                Image im = ImUtil.ChangePixelFormat(Image.FromFile(openFileDialog1.FileName, true),PixelFormat.Format24bppRgb); 
                if (im == null) throw new Exception();
                theFormImage.pictureBox1.Image = im;
                theFormImage.Text = System.IO.Path.GetFileNameWithoutExtension(openFileDialog1.FileName);
                theFormImage.Show();
                toolStripStatusLabel1.Text = 
                    String.Format("Image loaded, {0} x {1}, {2}",
                       theFormImage.pictureBox1.Image.Width,
                       theFormImage.pictureBox1.Image.Height,
                       theFormImage.pictureBox1.Image.PixelFormat);
                Cursor.Current = Cursors.Default;
            }
            catch
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Unable to open image file:\n" + openFileDialog1.FileName);
                theFormImage.Dispose();
            }

        }

        #region Routines to save files in various formats

        // For identifying a suitable codec.
        private static ImageCodecInfo GetEncoderInfo(String mimeType) 
        {
            int j;
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();
            for (j = 0; j < codecs.Length; j++)
            {
                if (codecs[j].MimeType == mimeType)
                {
                    return codecs[j];
                }
            }
            return null ;
        }

        // Several "save" routines, almost exactly alike, but left separate
        // so you can modify them to set more parameters

        private void SaveJpeg8()
        {
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                theFormImage.pictureBox1.Image.Save(saveFileDialog1.FileName, ImageFormat.Jpeg);
                Cursor.Current = Cursors.Default;
            }
            catch
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Unable to save the file.");
            }
        }

        private void SaveJpegHighQuality()
        {
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                // Get an ImageCodecInfo object that represents the JPEG codec.
                ImageCodecInfo codec = GetEncoderInfo("image/tiff");
                System.Drawing.Imaging.Encoder encoder = System.Drawing.Imaging.Encoder.Quality;
                EncoderParameters encoderParameters = new EncoderParameters(1);
                encoderParameters.Param[0] = new EncoderParameter(encoder, 100L);  // for "100%"
                theFormImage.pictureBox1.Image.Save(saveFileDialog1.FileName, codec, encoderParameters);
                Cursor.Current = Cursors.Default;
            }
            catch
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Unable to save the file.");
            }
        }

        private void SaveTiff8()
        {
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                theFormImage.pictureBox1.Image.Save(saveFileDialog1.FileName, ImageFormat.Tiff);
                Cursor.Current = Cursors.Default;
            }
            catch
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Unable to save the file.");
            }
        }

        
        #endregion

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {

            if (theFormImage == null)
            {
                MessageBox.Show("There is no image to save.");
                return;
            }

            // Choose a file name and type

            saveFileDialog1.FileName = theFormImage.Text + "X";

            saveFileDialog1.Filter = "JPEG 3 x 8-bit compressed|*.jpg|JPEG 3 x 8-bit uncompressed|*.jpg|TIFF 3 x 8-bit|*.tiff";
            
            DialogResult r = saveFileDialog1.ShowDialog();
            if (r != DialogResult.OK) return;

            if (saveFileDialog1.FilterIndex == 1)
            {
                SaveJpeg8();
            }
            else if (saveFileDialog1.FilterIndex == 2)
            {
                SaveJpegHighQuality();
            }
            else if (saveFileDialog1.FilterIndex == 3) 
            {
                SaveTiff8();
            }
            else
            {
                MessageBox.Show("Problem recognizing your file format choice.");
                return;
            }

            return;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((theUndoImage == null) || (theFormImage == null))
            {
                MessageBox.Show("'Undo' is not available.");
            }
            else
            {
                theFormImage.pictureBox1.Image = theUndoImage;
                theUndoImage = null;
            }
        }

        private void imageInformationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null)
            {
                MessageBox.Show("No image to analyze");
                return;
            }
            
            if (theFormReport != null) theFormReport.Dispose();
            theFormReport = new FormReport();
            theFormReport.MdiParent = this;
            theFormReport.Text = "Image Information";
            theFormReport.SetText(ImProc.ImageInfo(theFormImage.pictureBox1.Image));
            theFormReport.Show();
        }

        private void helpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "CovingtonImageProcessing - M. Covington 2007\r\n\r\n" +
                "This program is a framework for experimenting with your own image " +
                "processing algorithms in C#.  It can open images in almost any graphics format for which " +
                "which your PC has a codec, display them, and save them in a few specified formats.\r\n\r\n" +
                "Upon loading, every image is converted to 3 x 8-bit RGB, and a routine is provided" +
                "to convert it quickly to or from 3 arrays (R, G, B) of bytes, " +
                "or a single array for monochrome work.  The arrays are int[x,y], with [0,0] at the " +
                "upper left, with values from 0 to 255; when converted back to an image, values " +
                "outside that range are clipped to 0 or 255 respectively.\r\n\r\n" +
                "Class FormMain contains the user interface; ImUtil contains utility functions; " +
                "and ImProc contains the image processing algorithms.\r\n\r\n" +
                "To implement your own algorithm, add it to ImProc, imitating the examples " +
                "already there, and add an item to the menu bar in FormMain, again imitating " +
                "the examples already there.\r\n\r\n" +
                "FormMain has two MDI child forms, FormImage, for the image, and FormReport, " +
                "for textual messages displayed in a scrolling fixed-pitch text box.\r\n\r\n" +
                "Since this is a tool for exploratory programming, it makes relatively little use of " +
                "exception handling.  Instead, illegal operations result in a message box followed by " +
                "a no-op or program termination."
            );
        }

        #endregion

        #region User interface tools

        /// <summary>
        /// Replaces the image on theFormImage with a new one, preserving the
        /// old one for one level of undo.  Does nothing if given a null image.
        /// </summary>
        void ReplaceImage(Bitmap im)
        {
            if ((im == null) || (theFormImage == null)) return;
            theUndoImage = (Bitmap)theFormImage.pictureBox1.Image;
            theFormImage.pictureBox1.Image = im;
        }

        /// <summary>
        /// Sets the hourglass cursor AND starts timing the operation.
        /// </summary>
        void SetHourglassCursor()
        {
            Cursor.Current = Cursors.WaitCursor;
            theTime = DateTime.Now;
        }

        /// <summary>
        /// Clears the hourglass cursor AND reports time elapsed.
        /// </summary>
        void SetNormalCursor()
        {
            Cursor.Current = Cursors.Default;
            TimeSpan elapsed = DateTime.Now - theTime;
            toolStripStatusLabel1.Text = "Time elapsed: " + elapsed.ToString();
        }

        #endregion

        #region User-supplied processing functions

        private void makeMonochromeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.MakeMonochrome((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }

        private void invertToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.Invert((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }

        private void edgeFinderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.EdgeFinder((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }

        private void sharpenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.Sharpen((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }

        private void blurToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.Blur((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }

        private void smearVerticallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (theFormImage == null) return;
            SetHourglassCursor();
            ReplaceImage(ImProc.SmearVertically((Bitmap)theFormImage.pictureBox1.Image));
            SetNormalCursor();
        }


        // Add items here to access whatever functions you have added to ImProc.
        // All of them should use ReplaceImage so that Undo will work correctly.

        #endregion
    }
}