using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace CovingtonImageProcessing
{
    /// <summary>
    /// Image processing procedures and functions (all static).
    /// Rules of the game:
    /// (0) These are subject to change.
    /// (0) Regardless of its original format, the image is always seen as 0-255 R-G-B.
    /// (1) Return a modified copy of the image; do not operate on the original one.
    /// (2) If the operation cannot be done, return null; do not crash.
    /// (3) In this environment, processing a large image can be very slow.
    /// </summary>
    static class ImProc
    {

        /// <summary>
        /// Textual description of the image, for display in a fixed-pitch TextBox.
        /// </summary>
        public static string ImageInfo(Image im)
        {
            if (im == null) return "No image to process";

            return
                String.Format(
                  // Each has 20 characters before the value
                  "Height x width:     {0} x {1}" + Environment.NewLine +
                  "Pixel format:       {2}" + Environment.NewLine +
                  "V x H resolution:   {3} x {4}" + Environment.NewLine,
                  
                  im.Height, im.Width,
                  im.PixelFormat,
                  im.VerticalResolution, im.HorizontalResolution

                );
        }


        // Example of one simple manipulation
        public static Bitmap MakeMonochrome(Bitmap bm)
        {
            int[,] px;
            ImUtil.MonoImageToArray(bm, out px);
            ImUtil.MonoArrayToImage(px, out bm);
            return bm;
        }

        // Another almost-as-simple manipulation
        public static Bitmap Invert(Bitmap bm)
        {
            int[,] r, g, b;
            ImUtil.RgbImageToArrays(bm, out r, out g, out b);

            for (int x = 0; x < bm.Width; x++)
            {
                for (int y = 0; y < bm.Height; y++)
                {
                    r[x, y] = 255 - r[x, y];
                    g[x, y] = 255 - g[x, y];
                    b[x, y] = 255 - b[x, y];
                }
            }

            Bitmap bm2;
            ImUtil.RgbArraysToImage(r, g, b, out bm2);
            return bm2;
        }

        // Example of a convolution
        public static Bitmap EdgeFinder(Bitmap im)
        {
            int[,] kernel =   { { -1, -1, -1 },
                                    { -1,  8, -1 },
                                    { -1, -1, -1 } };

            return ImUtil.Convolve(kernel,im);
        }

        // Example of a convolution
        public static Bitmap Sharpen(Bitmap im)
        {
            // Like edge finder but middle number is much larger,
            // to preserve a substantial component of the original image

            int[,] kernel =   { { -1, -1, -1 },
                                    { -1, 20, -1 },
                                    { -1, -1, -1 } };

            return ImUtil.Convolve(kernel, im);
        }

        // Example of a convolution with a larger kernel
        public static Bitmap Blur(Bitmap im)
        {
            int[,] kernel =   { {  1,  2,  4,  2,  1 },
                                    {  2,  4,  8,  4,  2 },
                                    {  4,  8, 16,  8,  4 },
                                    {  2,  4,  8,  4,  2 },
                                    {  1,  2,  4,  2,  1 } };

            return ImUtil.Convolve(kernel, im);
        }

        // Example of a convolution in just one direction
        // (routine Convolve still requires square matrix;
        // a faster algorithm could be coded)
        public static Bitmap SmearVertically(Bitmap im)
        {
            int[,] kernel =   { {  0,  0,  1,  0,  0 },
                                    {  0,  0,  2,  0,  0 },
                                    {  0,  0,  4,  0,  0 },
                                    {  0,  0,  2,  0,  0 },
                                    {  0,  0,  1,  0,  0 } };

            return ImUtil.Convolve(kernel, im);
        }


    }
}
