using System;
using System.Drawing;
using System.Drawing.Imaging;

/// <summary>
/// Utility routines for image processing.
/// </summary>
public static class ImUtil
{
    /// <summary>
    /// Converts the image from any pixel format to the specified one by rendering it. 
    /// Cannot convert from indexed format.
    /// Cannot convert to Format16bppGrayScale or Format16bppArgb1555.
    /// (See Graphics.FromImage documentation for more about this.)
    /// Returns null if the conversion cannot be done.
    /// (Note that Bitmap is a subclass of Image.)
    /// </summary>
    public static Bitmap ChangePixelFormat(Image im,PixelFormat pf)
    {
        Bitmap b1=new Bitmap(im);
        Bitmap b2=new Bitmap(b1.Size.Width,b1.Size.Height,pf);
        Graphics g;
        try
        {
            g = Graphics.FromImage(b2);
        }
        catch
        {
            System.Windows.Forms.MessageBox.Show("Conversion from " + im.PixelFormat.ToString() + " to " + pf.ToString() + " is not supported.");
            return null;
        }
        g.DrawImage(b1,new Point(0,0));
        g.Dispose();
        return b2;
    }

    /// <summary>
    /// Given an image, converts it to 3 arrays (one each for red, green, and blue).
    /// Arrays are of type int[x,y] where [0,0] is the top left corner.
    /// </summary>
    /// <param name="im">Image to be processed; must be 48-bit RGB.</param>
    /// <param name="r">Array of red pixel values.</param>
    /// <param name="g">Array of green pixel values.</param>
    /// <param name="b">Array of blue pixel values.</param>
    public static void RgbImageToArrays(Bitmap bm, out int[,] r, out int[,] g, out int[,] b)
    {
        // Create arrays in which to return results (must be done before ANY return)
        r = new int[bm.Width, bm.Height];
        g = new int[bm.Width, bm.Height];
        b = new int[bm.Width, bm.Height];

        // Check valid input
        if (bm.PixelFormat != PixelFormat.Format24bppRgb)
        {
            System.Windows.Forms.MessageBox.Show("Cannot convert image to array - wrong format.");
            return;
        }

        // Copy the image into an array of B, G, R values
        Rectangle rect = new Rectangle(0, 0, bm.Width, bm.Height);
        BitmapData bd =  bm.LockBits(rect, ImageLockMode.ReadOnly, bm.PixelFormat);
        int bytes = bd.Stride * bd.Height;  // not bm.Width * bm.Height * 3;  
        byte[] bgrValues = new byte[bytes];
        IntPtr ptr = bd.Scan0;
        System.Runtime.InteropServices.Marshal.Copy(ptr, bgrValues, 0, bytes);
        bm.UnlockBits(bd);

        // Divide it up into red, green, and blue
        for (int x = 0; x < bm.Width; x++)
        {
            for (int y = 0; y < bm.Height; y++)
            {
                int j = x*3 + y*bd.Stride;
                b[x, y] = bgrValues[j];
                g[x, y] = bgrValues[j + 1];
                r[x, y] = bgrValues[j + 2];
            }
        }

        return;
    }

    /// <summary>
    /// Like RgbImageToArrays, but produces a single monochrome array containing the
    /// (approximate) average of the three color pixel values.
    /// </summary>
    public static void MonoImageToArray(Bitmap bm, out int[,] px)
    {
        int[,] r, g, b;
        RgbImageToArrays(bm, out r, out g, out b);
        int xsize = r.GetLength(0);
        int ysize = r.GetLength(1);
        px = new int[xsize, ysize];
        for (int x = 0; x < xsize; x++)
        {
            for (int y = 0; y < ysize; y++)
            {
                px[x, y] = ((r[x, y] + g[x, y] + b[x, y]) / 3);
            }
        }
        return;
    }

    /// <summary>
    /// Given 3 arrays, converts them to an image (a Bitmap).  Inverse of RgbImageToArrays.
    /// </summary>
    /// <param name="r">Array of red pixel values.</param>
    /// <param name="g">Array of green pixel values.</param>
    /// <param name="b">Array of blue pixel values.</param>
    /// <param name="bm">Image to be created; will be 3 x 8-bit RGB.</param>
    public static void RgbArraysToImage(int[,] r, int[,] g, int[,] b, out Bitmap bm)
    {
        // Create the bitmap
        bm = new Bitmap(r.GetLength(0), r.GetLength(1), PixelFormat.Format24bppRgb);

        // Create a byte array to match the bitmap
        Rectangle rect = new Rectangle(0, 0, bm.Width, bm.Height);
        BitmapData bd = bm.LockBits(rect, ImageLockMode.ReadOnly, bm.PixelFormat);
        int bytes = bd.Stride * bd.Height;  // not bm.Width * bm.Height * 3;  
        byte[] bgrValues = new byte[bytes];

        // Copy the pixel values into the byte array
        for (int x = 0; x < bm.Width; x++)
        {
            for (int y = 0; y < bm.Height; y++)
            {
                int j = x * 3 + y * bd.Stride;
                int bb = b[x, y]; 
                    if (bb < 0) bb = 0; 
                    if (bb > 255) bb = 255; 
                    bgrValues[j] = (byte)bb;
                int gg = g[x, y]; 
                    if (gg < 0) gg = 0; 
                    if (gg > 255) gg = 255; 
                    bgrValues[j+1] = (byte)gg;
                int rr = r[x, y]; 
                    if (rr < 0) rr = 0; 
                    if (rr > 255) rr = 255; 
                    bgrValues[j+2] = (byte)rr;
            }
        }

        // Copy the byte array into the bitmap and unlock it
        IntPtr ptr = bd.Scan0;
        System.Runtime.InteropServices.Marshal.Copy(bgrValues, 0, ptr, bytes);
        bm.UnlockBits(bd);

        return;
    }

    /// <summary>
    /// Given 1 arrays, converts it to an image (a Bitmap).  Inverse of MonoImageToArrays.
    /// </summary>
    /// <param name="px">Pixel array, int[x,y], where [0,0] is upper left.</param>
    /// <param name="bm">Image to be created; will be 3 x 8-bit RGB.</param>
    public static void MonoArrayToImage(int[,] px, out Bitmap bm)
    {
        RgbArraysToImage(px, px, px, out bm);
    }


    /// <summary>
    /// General routine for convolutions.
    /// NOTE: For ease of writing in C#, the kernel has rows and columns
    /// in [y,x] order.  See examples.
    /// </summary>
    public static Bitmap Convolve(int[,] kernel, Bitmap bm)
    {
        int ksize = kernel.GetLength(0);  // assumed to be square
        int kradius = ksize / 2;

        // Compute kernel sum
        int ksum = 0;
        for (int x = 0; x < ksize; x++)
            for (int y = 0; y < ksize; y++)
                ksum += kernel[y, x];
        if (ksum == 0) ksum = 1;

        // Obtain arrays containing original image
        int[,] r, g, b;
        RgbImageToArrays(bm, out r, out g, out b);

        // New arrays to hold new image, initialized to zero
        int[,] rr, gg, bb;
        rr = new int[bm.Width, bm.Height];
        gg = new int[bm.Width, bm.Height];
        bb = new int[bm.Width, bm.Height];

        // Work over the whole image
        for (int x = kradius; x < bm.Width - kradius; x++)
            for (int y = kradius; y < bm.Height - kradius; y++)
            {
                // Convolve red, green, and blue
                for (int dx = -kradius; dx <= kradius; dx++)
                    for (int dy = -kradius; dy <= kradius; dy++)
                    {
                        // Rely on the optimizing compiler to use temp variables here
                        rr[x,y] += kernel[kradius + dy, kradius + dx] * r[x + dx, y + dy];
                        gg[x,y] += kernel[kradius + dy, kradius + dx] * g[x + dx, y + dy];
                        bb[x,y] += kernel[kradius + dy, kradius + dx] * b[x + dx, y + dy];
                    }
                // Scale values 
                rr[x,y] /= ksum;
                gg[x,y] /= ksum;
                bb[x,y] /= ksum;
            }

        Bitmap bmnew;
        RgbArraysToImage(rr, gg, bb, out bmnew);
        return bmnew;
    }

}


// TO DO NEXT:  ArraysToImage.
// The monochrome ArrayToImage is then just ArraysToImage(px,px,px,bm).
